@@ -33,6 +33,7 @@
 #include "ilo_context.h"
 #include "ilo_cp.h"
 #include "ilo_gpe_gen6.h"
+#include "ilo_gpe_gen7.h"
 #include "ilo_shader.h"
 #include "ilo_state.h"
 #include "ilo_3d_pipeline.h"
@@ -64,7 +65,7 @@ gen6_wa_pipe_control_post_sync(struct ilo_3d_pipeline *p,
     *
     * The workaround below necessitates this workaround.
     */
-   p->gen6_PIPE_CONTROL(p->dev,
+   gen6_emit_PIPE_CONTROL(p->dev,
          PIPE_CONTROL_CS_STALL |
          PIPE_CONTROL_STALL_AT_SCOREBOARD,
          NULL, 0, false, p->cp);
@@ -83,7 +84,7 @@ gen6_wa_pipe_control_post_sync(struct ilo_3d_pipeline *p,
     *     "Before a PIPE_CONTROL with Write Cache Flush Enable =1, a
     *      PIPE_CONTROL with any non-zero post-sync-op is required."
     */
-   p->gen6_PIPE_CONTROL(p->dev,
+   gen6_emit_PIPE_CONTROL(p->dev,
          PIPE_CONTROL_WRITE_IMMEDIATE,
          p->workaround_bo, 0, false, p->cp);
 }
@@ -103,7 +104,7 @@ gen6_wa_pipe_control_wm_multisample_flush(struct ilo_3d_pipeline *p)
     *      requires driver to send a PIPE_CONTROL with a CS stall along with a
     *      Depth Flush prior to this command."
     */
-   p->gen6_PIPE_CONTROL(p->dev,
+   gen6_emit_PIPE_CONTROL(p->dev,
          PIPE_CONTROL_DEPTH_CACHE_FLUSH |
          PIPE_CONTROL_CS_STALL,
          0, 0, false, p->cp);
@@ -121,15 +122,15 @@ gen6_wa_pipe_control_wm_depth_flush(struct ilo_3d_pipeline *p)
     * to emit a sequence of PIPE_CONTROLs prior to emitting depth related
     * commands.
     */
-   p->gen6_PIPE_CONTROL(p->dev,
+   gen6_emit_PIPE_CONTROL(p->dev,
          PIPE_CONTROL_DEPTH_STALL,
          NULL, 0, false, p->cp);
 
-   p->gen6_PIPE_CONTROL(p->dev,
+   gen6_emit_PIPE_CONTROL(p->dev,
          PIPE_CONTROL_DEPTH_CACHE_FLUSH,
          NULL, 0, false, p->cp);
 
-   p->gen6_PIPE_CONTROL(p->dev,
+   gen6_emit_PIPE_CONTROL(p->dev,
          PIPE_CONTROL_DEPTH_STALL,
          NULL, 0, false, p->cp);
 }
@@ -150,7 +151,7 @@ gen6_wa_pipe_control_wm_max_threads_stall(struct ilo_3d_pipeline *p)
     *      field set (DW1 Bit 1), must be issued prior to any change to the
     *      value in this field (Maximum Number of Threads in 3DSTATE_WM)"
     */
-   p->gen6_PIPE_CONTROL(p->dev,
+   gen6_emit_PIPE_CONTROL(p->dev,
          PIPE_CONTROL_STALL_AT_SCOREBOARD,
          NULL, 0, false, p->cp);
 
@@ -168,7 +169,7 @@ gen6_wa_pipe_control_vs_const_flush(struct ilo_3d_pipeline *p)
     * PIPE_CONTROL after 3DSTATE_CONSTANT_VS so that the command is kept being
     * buffered by VS FF, to the point that the FF dies.
     */
-   p->gen6_PIPE_CONTROL(p->dev,
+   gen6_emit_PIPE_CONTROL(p->dev,
          PIPE_CONTROL_DEPTH_STALL |
          PIPE_CONTROL_INSTRUCTION_FLUSH |
          PIPE_CONTROL_STATE_CACHE_INVALIDATE,
@@ -187,7 +188,7 @@ gen6_pipeline_common_select(struct ilo_3d_pipeline *p,
       if (p->dev->gen == ILO_GEN(6))
          gen6_wa_pipe_control_post_sync(p, false);
 
-      p->gen6_PIPELINE_SELECT(p->dev, 0x0, p->cp);
+      gen6_emit_PIPELINE_SELECT(p->dev, 0x0, p->cp);
    }
 }
 
@@ -201,7 +202,7 @@ gen6_pipeline_common_sip(struct ilo_3d_pipeline *p,
       if (p->dev->gen == ILO_GEN(6))
          gen6_wa_pipe_control_post_sync(p, false);
 
-      p->gen6_STATE_SIP(p->dev, 0, p->cp);
+      gen6_emit_STATE_SIP(p->dev, 0, p->cp);
    }
 }
 
@@ -216,7 +217,7 @@ gen6_pipeline_common_base_address(struct ilo_3d_pipeline *p,
       if (p->dev->gen == ILO_GEN(6))
          gen6_wa_pipe_control_post_sync(p, false);
 
-      p->gen6_STATE_BASE_ADDRESS(p->dev,
+      gen6_emit_STATE_BASE_ADDRESS(p->dev,
             NULL, p->cp->bo, p->cp->bo, NULL, ilo->hw3d->kernel.bo,
             0, 0, 0, 0, p->cp);
 
@@ -312,7 +313,7 @@ gen6_pipeline_common_urb(struct ilo_3d_pipeline *p,
          gs_total_size = 0;
       }
 
-      p->gen6_3DSTATE_URB(p->dev, vs_total_size, gs_total_size,
+      gen6_emit_3DSTATE_URB(p->dev, vs_total_size, gs_total_size,
             vs_entry_size, gs_entry_size, p->cp);
 
       /*
@@ -338,7 +339,7 @@ gen6_pipeline_common_pointers_1(struct ilo_3d_pipeline *p,
 {
    /* 3DSTATE_VIEWPORT_STATE_POINTERS */
    if (session->viewport_state_changed) {
-      p->gen6_3DSTATE_VIEWPORT_STATE_POINTERS(p->dev,
+      gen6_emit_3DSTATE_VIEWPORT_STATE_POINTERS(p->dev,
             p->state.CLIP_VIEWPORT,
             p->state.SF_VIEWPORT,
             p->state.CC_VIEWPORT, p->cp);
@@ -354,7 +355,7 @@ gen6_pipeline_common_pointers_2(struct ilo_3d_pipeline *p,
    if (session->cc_state_blend_changed ||
        session->cc_state_dsa_changed ||
        session->cc_state_cc_changed) {
-      p->gen6_3DSTATE_CC_STATE_POINTERS(p->dev,
+      gen6_emit_3DSTATE_CC_STATE_POINTERS(p->dev,
             p->state.BLEND_STATE,
             p->state.DEPTH_STENCIL_STATE,
             p->state.COLOR_CALC_STATE, p->cp);
@@ -364,7 +365,7 @@ gen6_pipeline_common_pointers_2(struct ilo_3d_pipeline *p,
    if (session->sampler_state_vs_changed ||
        session->sampler_state_gs_changed ||
        session->sampler_state_fs_changed) {
-      p->gen6_3DSTATE_SAMPLER_STATE_POINTERS(p->dev,
+      gen6_emit_3DSTATE_SAMPLER_STATE_POINTERS(p->dev,
             p->state.vs.SAMPLER_STATE,
             0,
             p->state.wm.SAMPLER_STATE, p->cp);
@@ -378,7 +379,7 @@ gen6_pipeline_common_pointers_3(struct ilo_3d_pipeline *p,
 {
    /* 3DSTATE_SCISSOR_STATE_POINTERS */
    if (session->scissor_state_changed) {
-      p->gen6_3DSTATE_SCISSOR_STATE_POINTERS(p->dev,
+      gen6_emit_3DSTATE_SCISSOR_STATE_POINTERS(p->dev,
             p->state.SCISSOR_RECT, p->cp);
    }
 
@@ -386,7 +387,7 @@ gen6_pipeline_common_pointers_3(struct ilo_3d_pipeline *p,
    if (session->binding_table_vs_changed ||
        session->binding_table_gs_changed ||
        session->binding_table_fs_changed) {
-      p->gen6_3DSTATE_BINDING_TABLE_POINTERS(p->dev,
+      gen6_emit_3DSTATE_BINDING_TABLE_POINTERS(p->dev,
             p->state.vs.BINDING_TABLE_STATE,
             p->state.gs.BINDING_TABLE_STATE,
             p->state.wm.BINDING_TABLE_STATE, p->cp);
@@ -398,18 +399,31 @@ gen6_pipeline_vf(struct ilo_3d_pipeline *p,
                  const struct ilo_context *ilo,
                  struct gen6_pipeline_session *session)
 {
-   /* 3DSTATE_INDEX_BUFFER */
-   if (DIRTY(IB) || session->primitive_restart_changed ||
-       session->batch_bo_changed) {
-      p->gen6_3DSTATE_INDEX_BUFFER(p->dev,
-            &ilo->ib, ilo->draw->primitive_restart, p->cp);
+   if (p->dev->gen >= ILO_GEN(7.5)) {
+      /* 3DSTATE_INDEX_BUFFER */
+      if (DIRTY(IB) || session->batch_bo_changed) {
+         gen6_emit_3DSTATE_INDEX_BUFFER(p->dev,
+               &ilo->ib, false, p->cp);
+      }
+
+      /* 3DSTATE_VF */
+      if (session->primitive_restart_changed) {
+         gen7_emit_3DSTATE_VF(p->dev, ilo->draw->primitive_restart,
+               ilo->draw->restart_index, p->cp);
+      }
+   }
+   else {
+      /* 3DSTATE_INDEX_BUFFER */
+      if (DIRTY(IB) || session->primitive_restart_changed ||
+          session->batch_bo_changed) {
+         gen6_emit_3DSTATE_INDEX_BUFFER(p->dev,
+               &ilo->ib, ilo->draw->primitive_restart, p->cp);
+      }
    }
 
    /* 3DSTATE_VERTEX_BUFFERS */
-   if (DIRTY(VB) || DIRTY(VE) || session->batch_bo_changed) {
-      p->gen6_3DSTATE_VERTEX_BUFFERS(p->dev,
-            ilo->vb.states, ilo->vb.enabled_mask, ilo->ve, p->cp);
-   }
+   if (DIRTY(VB) || DIRTY(VE) || session->batch_bo_changed)
+      gen6_emit_3DSTATE_VERTEX_BUFFERS(p->dev, ilo->ve, &ilo->vb, p->cp);
 
    /* 3DSTATE_VERTEX_ELEMENTS */
    if (DIRTY(VE) || DIRTY(VS)) {
@@ -434,7 +448,7 @@ gen6_pipeline_vf(struct ilo_3d_pipeline *p,
             prepend_generate_ids = true;
       }
 
-      p->gen6_3DSTATE_VERTEX_ELEMENTS(p->dev, ve,
+      gen6_emit_3DSTATE_VERTEX_ELEMENTS(p->dev, ve,
             last_velement_edgeflag, prepend_generate_ids, p->cp);
    }
 }
@@ -446,16 +460,16 @@ gen6_pipeline_vf_statistics(struct ilo_3d_pipeline *p,
 {
    /* 3DSTATE_VF_STATISTICS */
    if (session->hw_ctx_changed)
-      p->gen6_3DSTATE_VF_STATISTICS(p->dev, false, p->cp);
+      gen6_emit_3DSTATE_VF_STATISTICS(p->dev, false, p->cp);
 }
 
-void
+static void
 gen6_pipeline_vf_draw(struct ilo_3d_pipeline *p,
                       const struct ilo_context *ilo,
                       struct gen6_pipeline_session *session)
 {
    /* 3DPRIMITIVE */
-   p->gen6_3DPRIMITIVE(p->dev, ilo->draw, &ilo->ib, false, p->cp);
+   gen6_emit_3DPRIMITIVE(p->dev, ilo->draw, &ilo->ib, false, p->cp);
    p->state.has_gen6_wa_pipe_control = false;
 }
 
@@ -477,7 +491,7 @@ gen6_pipeline_vs(struct ilo_3d_pipeline *p,
 
    /* 3DSTATE_CONSTANT_VS */
    if (emit_3dstate_constant_vs) {
-      p->gen6_3DSTATE_CONSTANT_VS(p->dev,
+      gen6_emit_3DSTATE_CONSTANT_VS(p->dev,
             &p->state.vs.PUSH_CONSTANT_BUFFER,
             &p->state.vs.PUSH_CONSTANT_BUFFER_size,
             1, p->cp);
@@ -487,7 +501,7 @@ gen6_pipeline_vs(struct ilo_3d_pipeline *p,
    if (emit_3dstate_vs) {
       const int num_samplers = ilo->sampler[PIPE_SHADER_VERTEX].count;
 
-      p->gen6_3DSTATE_VS(p->dev, ilo->vs, num_samplers, p->cp);
+      gen6_emit_3DSTATE_VS(p->dev, ilo->vs, num_samplers, p->cp);
    }
 
    if (emit_3dstate_constant_vs && p->dev->gen == ILO_GEN(6))
@@ -501,14 +515,14 @@ gen6_pipeline_gs(struct ilo_3d_pipeline *p,
 {
    /* 3DSTATE_CONSTANT_GS */
    if (session->pcb_state_gs_changed)
-      p->gen6_3DSTATE_CONSTANT_GS(p->dev, NULL, NULL, 0, p->cp);
+      gen6_emit_3DSTATE_CONSTANT_GS(p->dev, NULL, NULL, 0, p->cp);
 
    /* 3DSTATE_GS */
    if (DIRTY(GS) || DIRTY(VS) ||
        session->prim_changed || session->kernel_bo_changed) {
       const int verts_per_prim = u_vertices_per_prim(session->reduced_prim);
 
-      p->gen6_3DSTATE_GS(p->dev, ilo->gs, ilo->vs, verts_per_prim, p->cp);
+      gen6_emit_3DSTATE_GS(p->dev, ilo->gs, ilo->vs, verts_per_prim, p->cp);
    }
 }
 
@@ -568,7 +582,7 @@ gen6_pipeline_gs_svbi(struct ilo_3d_pipeline *p,
       if (p->dev->gen == ILO_GEN(6))
          gen6_wa_pipe_control_post_sync(p, false);
 
-      p->gen6_3DSTATE_GS_SVB_INDEX(p->dev,
+      gen6_emit_3DSTATE_GS_SVB_INDEX(p->dev,
             0, p->state.so_num_vertices, p->state.so_max_vertices,
             false, p->cp);
 
@@ -585,7 +599,7 @@ gen6_pipeline_gs_svbi(struct ilo_3d_pipeline *p,
           *      0xFFFFFFFF in order to not cause overflow in that SVBI."
           */
          for (i = 1; i < 4; i++) {
-            p->gen6_3DSTATE_GS_SVB_INDEX(p->dev,
+            gen6_emit_3DSTATE_GS_SVB_INDEX(p->dev,
                   i, 0, 0xffffffff, false, p->cp);
          }
       }
@@ -616,7 +630,7 @@ gen6_pipeline_clip(struct ilo_3d_pipeline *p,
          }
       }
 
-      p->gen6_3DSTATE_CLIP(p->dev, ilo->rasterizer,
+      gen6_emit_3DSTATE_CLIP(p->dev, ilo->rasterizer,
             ilo->fs, enable_guardband, 1, p->cp);
    }
 }
@@ -627,10 +641,8 @@ gen6_pipeline_sf(struct ilo_3d_pipeline *p,
                  struct gen6_pipeline_session *session)
 {
    /* 3DSTATE_SF */
-   if (DIRTY(RASTERIZER) || DIRTY(VS) || DIRTY(GS) || DIRTY(FS)) {
-      p->gen6_3DSTATE_SF(p->dev, ilo->rasterizer, ilo->fs,
-            (ilo->gs) ? ilo->gs : ilo->vs, p->cp);
-   }
+   if (DIRTY(RASTERIZER) || DIRTY(FS))
+      gen6_emit_3DSTATE_SF(p->dev, ilo->rasterizer, ilo->fs, p->cp);
 }
 
 void
@@ -643,7 +655,7 @@ gen6_pipeline_sf_rect(struct ilo_3d_pipeline *p,
       if (p->dev->gen == ILO_GEN(6))
          gen6_wa_pipe_control_post_sync(p, false);
 
-      p->gen6_3DSTATE_DRAWING_RECTANGLE(p->dev, 0, 0,
+      gen6_emit_3DSTATE_DRAWING_RECTANGLE(p->dev, 0, 0,
             ilo->fb.state.width, ilo->fb.state.height, p->cp);
    }
 }
@@ -654,21 +666,25 @@ gen6_pipeline_wm(struct ilo_3d_pipeline *p,
                  struct gen6_pipeline_session *session)
 {
    /* 3DSTATE_CONSTANT_PS */
-   if (session->pcb_state_fs_changed)
-      p->gen6_3DSTATE_CONSTANT_PS(p->dev, NULL, NULL, 0, p->cp);
+   if (session->pcb_state_fs_changed) {
+      gen6_emit_3DSTATE_CONSTANT_PS(p->dev,
+            &p->state.wm.PUSH_CONSTANT_BUFFER,
+            &p->state.wm.PUSH_CONSTANT_BUFFER_size,
+            1, p->cp);
+   }
 
    /* 3DSTATE_WM */
    if (DIRTY(FS) || DIRTY(SAMPLER_FS) || DIRTY(BLEND) || DIRTY(DSA) ||
        DIRTY(RASTERIZER) || session->kernel_bo_changed) {
       const int num_samplers = ilo->sampler[PIPE_SHADER_FRAGMENT].count;
       const bool dual_blend = ilo->blend->dual_blend;
-      const bool cc_may_kill = (ilo->dsa->alpha.enabled ||
+      const bool cc_may_kill = (ilo->dsa->dw_alpha ||
                                 ilo->blend->alpha_to_coverage);
 
       if (p->dev->gen == ILO_GEN(6) && session->hw_ctx_changed)
          gen6_wa_pipe_control_wm_max_threads_stall(p);
 
-      p->gen6_3DSTATE_WM(p->dev, ilo->fs, num_samplers,
+      gen6_emit_3DSTATE_WM(p->dev, ilo->fs, num_samplers,
             ilo->rasterizer, dual_blend, cc_may_kill, p->cp);
    }
 }
@@ -690,11 +706,11 @@ gen6_pipeline_wm_multisample(struct ilo_3d_pipeline *p,
          gen6_wa_pipe_control_wm_multisample_flush(p);
       }
 
-      p->gen6_3DSTATE_MULTISAMPLE(p->dev,
+      gen6_emit_3DSTATE_MULTISAMPLE(p->dev,
             ilo->fb.num_samples, packed_sample_pos,
             ilo->rasterizer->state.half_pixel_center, p->cp);
 
-      p->gen6_3DSTATE_SAMPLE_MASK(p->dev,
+      gen6_emit_3DSTATE_SAMPLE_MASK(p->dev,
             (ilo->fb.num_samples > 1) ? ilo->sample_mask : 0x1, p->cp);
    }
 }
@@ -724,10 +740,10 @@ gen6_pipeline_wm_depth(struct ilo_3d_pipeline *p,
          gen6_wa_pipe_control_wm_depth_flush(p);
       }
 
-      p->gen6_3DSTATE_DEPTH_BUFFER(p->dev, zs, p->cp);
+      gen6_emit_3DSTATE_DEPTH_BUFFER(p->dev, zs, p->cp);
 
       /* TODO */
-      p->gen6_3DSTATE_CLEAR_PARAMS(p->dev, 0, p->cp);
+      gen6_emit_3DSTATE_CLEAR_PARAMS(p->dev, 0, p->cp);
    }
 }
 
@@ -742,10 +758,10 @@ gen6_pipeline_wm_raster(struct ilo_3d_pipeline *p,
       if (p->dev->gen == ILO_GEN(6))
          gen6_wa_pipe_control_post_sync(p, false);
 
-      p->gen6_3DSTATE_POLY_STIPPLE_PATTERN(p->dev,
+      gen6_emit_3DSTATE_POLY_STIPPLE_PATTERN(p->dev,
             &ilo->poly_stipple, p->cp);
 
-      p->gen6_3DSTATE_POLY_STIPPLE_OFFSET(p->dev, 0, 0, p->cp);
+      gen6_emit_3DSTATE_POLY_STIPPLE_OFFSET(p->dev, 0, 0, p->cp);
    }
 
    /* 3DSTATE_LINE_STIPPLE */
@@ -753,7 +769,7 @@ gen6_pipeline_wm_raster(struct ilo_3d_pipeline *p,
       if (p->dev->gen == ILO_GEN(6))
          gen6_wa_pipe_control_post_sync(p, false);
 
-      p->gen6_3DSTATE_LINE_STIPPLE(p->dev,
+      gen6_emit_3DSTATE_LINE_STIPPLE(p->dev,
             ilo->rasterizer->state.line_stipple_pattern,
             ilo->rasterizer->state.line_stipple_factor + 1, p->cp);
    }
@@ -763,7 +779,7 @@ gen6_pipeline_wm_raster(struct ilo_3d_pipeline *p,
       if (p->dev->gen == ILO_GEN(6))
          gen6_wa_pipe_control_post_sync(p, false);
 
-      p->gen6_3DSTATE_AA_LINE_PARAMETERS(p->dev, p->cp);
+      gen6_emit_3DSTATE_AA_LINE_PARAMETERS(p->dev, p->cp);
    }
 }
 
@@ -774,23 +790,23 @@ gen6_pipeline_state_viewports(struct ilo_3d_pipeline *p,
 {
    /* SF_CLIP_VIEWPORT and CC_VIEWPORT */
    if (p->dev->gen >= ILO_GEN(7) && DIRTY(VIEWPORT)) {
-      p->state.SF_CLIP_VIEWPORT = p->gen7_SF_CLIP_VIEWPORT(p->dev,
+      p->state.SF_CLIP_VIEWPORT = gen7_emit_SF_CLIP_VIEWPORT(p->dev,
             ilo->viewport.cso, ilo->viewport.count, p->cp);
 
-      p->state.CC_VIEWPORT = p->gen6_CC_VIEWPORT(p->dev,
+      p->state.CC_VIEWPORT = gen6_emit_CC_VIEWPORT(p->dev,
             ilo->viewport.cso, ilo->viewport.count, p->cp);
 
       session->viewport_state_changed = true;
    }
    /* SF_VIEWPORT, CLIP_VIEWPORT, and CC_VIEWPORT */
    else if (DIRTY(VIEWPORT)) {
-      p->state.CLIP_VIEWPORT = p->gen6_CLIP_VIEWPORT(p->dev,
+      p->state.CLIP_VIEWPORT = gen6_emit_CLIP_VIEWPORT(p->dev,
             ilo->viewport.cso, ilo->viewport.count, p->cp);
 
-      p->state.SF_VIEWPORT = p->gen6_SF_VIEWPORT(p->dev,
+      p->state.SF_VIEWPORT = gen6_emit_SF_VIEWPORT(p->dev,
             ilo->viewport.cso, ilo->viewport.count, p->cp);
 
-      p->state.CC_VIEWPORT = p->gen6_CC_VIEWPORT(p->dev,
+      p->state.CC_VIEWPORT = gen6_emit_CC_VIEWPORT(p->dev,
             ilo->viewport.cso, ilo->viewport.count, p->cp);
 
       session->viewport_state_changed = true;
@@ -804,8 +820,8 @@ gen6_pipeline_state_cc(struct ilo_3d_pipeline *p,
 {
    /* BLEND_STATE */
    if (DIRTY(BLEND) || DIRTY(FB) || DIRTY(DSA)) {
-      p->state.BLEND_STATE = p->gen6_BLEND_STATE(p->dev,
-            ilo->blend, &ilo->fb, &ilo->dsa->alpha, p->cp);
+      p->state.BLEND_STATE = gen6_emit_BLEND_STATE(p->dev,
+            ilo->blend, &ilo->fb, ilo->dsa, p->cp);
 
       session->cc_state_blend_changed = true;
    }
@@ -813,8 +829,8 @@ gen6_pipeline_state_cc(struct ilo_3d_pipeline *p,
    /* COLOR_CALC_STATE */
    if (DIRTY(DSA) || DIRTY(STENCIL_REF) || DIRTY(BLEND_COLOR)) {
       p->state.COLOR_CALC_STATE =
-         p->gen6_COLOR_CALC_STATE(p->dev, &ilo->stencil_ref,
-               ilo->dsa->alpha.ref_value, &ilo->blend_color, p->cp);
+         gen6_emit_COLOR_CALC_STATE(p->dev, &ilo->stencil_ref,
+               ilo->dsa->alpha_ref, &ilo->blend_color, p->cp);
 
       session->cc_state_cc_changed = true;
    }
@@ -822,7 +838,7 @@ gen6_pipeline_state_cc(struct ilo_3d_pipeline *p,
    /* DEPTH_STENCIL_STATE */
    if (DIRTY(DSA)) {
       p->state.DEPTH_STENCIL_STATE =
-         p->gen6_DEPTH_STENCIL_STATE(p->dev, ilo->dsa, p->cp);
+         gen6_emit_DEPTH_STENCIL_STATE(p->dev, ilo->dsa, p->cp);
 
       session->cc_state_dsa_changed = true;
    }
@@ -836,7 +852,7 @@ gen6_pipeline_state_scissors(struct ilo_3d_pipeline *p,
    /* SCISSOR_RECT */
    if (DIRTY(SCISSOR) || DIRTY(VIEWPORT)) {
       /* there should be as many scissors as there are viewports */
-      p->state.SCISSOR_RECT = p->gen6_SCISSOR_RECT(p->dev,
+      p->state.SCISSOR_RECT = gen6_emit_SCISSOR_RECT(p->dev,
             &ilo->scissor, ilo->viewport.count, p->cp);
 
       session->scissor_state_changed = true;
@@ -861,7 +877,7 @@ gen6_pipeline_state_surfaces_rt(struct ilo_3d_pipeline *p,
 
          assert(surface && surface->is_rt);
          surface_state[i] =
-            p->gen6_SURFACE_STATE(p->dev, &surface->u.rt, true, p->cp);
+            gen6_emit_SURFACE_STATE(p->dev, &surface->u.rt, true, p->cp);
       }
 
       /*
@@ -876,7 +892,7 @@ gen6_pipeline_state_surfaces_rt(struct ilo_3d_pipeline *p,
                1, 0, &null_surface);
 
          surface_state[i] =
-            p->gen6_SURFACE_STATE(p->dev, &null_surface, true, p->cp);
+            gen6_emit_SURFACE_STATE(p->dev, &null_surface, true, p->cp);
 
          i++;
       }
@@ -915,7 +931,7 @@ gen6_pipeline_state_surfaces_so(struct ilo_3d_pipeline *p,
             (target < so->count) ? so->states[target] : NULL;
 
          if (so_target) {
-            surface_state[i] = p->gen6_so_SURFACE_STATE(p->dev,
+            surface_state[i] = gen6_emit_so_SURFACE_STATE(p->dev,
                   so_target, so_info, i, p->cp);
          }
          else {
@@ -981,7 +997,7 @@ gen6_pipeline_state_surfaces_view(struct ilo_3d_pipeline *p,
             (const struct ilo_view_cso *) view->states[i];
 
          surface_state[i] =
-            p->gen6_SURFACE_STATE(p->dev, &cso->surface, false, p->cp);
+            gen6_emit_SURFACE_STATE(p->dev, &cso->surface, false, p->cp);
       }
       else {
          surface_state[i] = 0;
@@ -1002,45 +1018,39 @@ gen6_pipeline_state_surfaces_const(struct ilo_3d_pipeline *p,
 {
    const struct ilo_cbuf_state *cbuf = &ilo->cbuf[shader_type];
    uint32_t *surface_state;
+   bool *binding_table_changed;
    int offset, count, i;
-   bool skip = false;
+
+   if (!DIRTY(CBUF))
+      return;
 
    /* SURFACE_STATEs for constant buffers */
    switch (shader_type) {
    case PIPE_SHADER_VERTEX:
-      if (DIRTY(CBUF)) {
-         offset = ILO_VS_CONST_SURFACE(0);
-         surface_state = &p->state.vs.SURFACE_STATE[offset];
-
-         session->binding_table_vs_changed = true;
-      }
-      else {
-         skip = true;
-      }
+      offset = ILO_VS_CONST_SURFACE(0);
+      surface_state = &p->state.vs.SURFACE_STATE[offset];
+      binding_table_changed = &session->binding_table_vs_changed;
       break;
    case PIPE_SHADER_FRAGMENT:
-      if (DIRTY(CBUF)) {
-         offset = ILO_WM_CONST_SURFACE(0);
-         surface_state = &p->state.wm.SURFACE_STATE[offset];
-
-         session->binding_table_fs_changed = true;
-      }
-      else {
-         skip = true;
-      }
+      offset = ILO_WM_CONST_SURFACE(0);
+      surface_state = &p->state.wm.SURFACE_STATE[offset];
+      binding_table_changed = &session->binding_table_fs_changed;
       break;
    default:
-      skip = true;
+      return;
       break;
    }
 
-   if (skip)
+   /* constants are pushed via PCB */
+   if (cbuf->enabled_mask == 0x1 && !cbuf->cso[0].resource) {
+      memset(surface_state, 0, ILO_MAX_CONST_BUFFERS * 4);
       return;
+   }
 
    count = util_last_bit(cbuf->enabled_mask);
    for (i = 0; i < count; i++) {
       if (cbuf->cso[i].resource) {
-         surface_state[i] = p->gen6_SURFACE_STATE(p->dev,
+         surface_state[i] = gen6_emit_SURFACE_STATE(p->dev,
                &cbuf->cso[i].surface, false, p->cp);
       }
       else {
@@ -1052,6 +1062,8 @@ gen6_pipeline_state_surfaces_const(struct ilo_3d_pipeline *p,
 
    if (count && session->num_surfaces[shader_type] < offset + count)
       session->num_surfaces[shader_type] = offset + count;
+
+   *binding_table_changed = true;
 }
 
 static void
@@ -1108,7 +1120,7 @@ gen6_pipeline_state_binding_tables(struct ilo_3d_pipeline *p,
    if (size < session->num_surfaces[shader_type])
       size = session->num_surfaces[shader_type];
 
-   *binding_table_state = p->gen6_BINDING_TABLE_STATE(p->dev,
+   *binding_table_state = gen6_emit_BINDING_TABLE_STATE(p->dev,
          surface_state, size, p->cp);
    *binding_table_state_size = size;
 }
@@ -1172,13 +1184,13 @@ gen6_pipeline_state_samplers(struct ilo_3d_pipeline *p,
 
       for (i = 0; i < num_samplers; i++) {
          border_color_state[i] = (samplers[i]) ?
-            p->gen6_SAMPLER_BORDER_COLOR_STATE(p->dev,
+            gen6_emit_SAMPLER_BORDER_COLOR_STATE(p->dev,
                   samplers[i], p->cp) : 0;
       }
    }
 
    /* should we take the minimum of num_samplers and num_views? */
-   *sampler_state = p->gen6_SAMPLER_STATE(p->dev,
+   *sampler_state = gen6_emit_SAMPLER_STATE(p->dev,
          samplers, views,
          border_color_state,
          MIN2(num_samplers, num_views), p->cp);
@@ -1190,27 +1202,83 @@ gen6_pipeline_state_pcb(struct ilo_3d_pipeline *p,
                         struct gen6_pipeline_session *session)
 {
    /* push constant buffer for VS */
-   if (DIRTY(VS) || DIRTY(CLIP)) {
+   if (DIRTY(VS) || DIRTY(CBUF) || DIRTY(CLIP)) {
+      const int cbuf0_size = (ilo->vs) ?
+            ilo_shader_get_kernel_param(ilo->vs,
+                  ILO_KERNEL_PCB_CBUF0_SIZE) : 0;
       const int clip_state_size = (ilo->vs) ?
             ilo_shader_get_kernel_param(ilo->vs,
                   ILO_KERNEL_VS_PCB_UCP_SIZE) : 0;
+      const int total_size = cbuf0_size + clip_state_size;
 
-      if (clip_state_size) {
+      if (total_size) {
          void *pcb;
 
-         p->state.vs.PUSH_CONSTANT_BUFFER_size = clip_state_size;
          p->state.vs.PUSH_CONSTANT_BUFFER =
-            p->gen6_push_constant_buffer(p->dev,
-                  p->state.vs.PUSH_CONSTANT_BUFFER_size, &pcb, p->cp);
+            gen6_emit_push_constant_buffer(p->dev, total_size, &pcb, p->cp);
+         p->state.vs.PUSH_CONSTANT_BUFFER_size = total_size;
 
-         memcpy(pcb, &ilo->clip, clip_state_size);
+         if (cbuf0_size) {
+            const struct ilo_cbuf_state *cbuf =
+               &ilo->cbuf[PIPE_SHADER_VERTEX];
+
+            if (cbuf0_size <= cbuf->cso[0].user_buffer_size) {
+               memcpy(pcb, cbuf->cso[0].user_buffer, cbuf0_size);
+            }
+            else {
+               memcpy(pcb, cbuf->cso[0].user_buffer,
+                     cbuf->cso[0].user_buffer_size);
+               memset(pcb + cbuf->cso[0].user_buffer_size, 0,
+                     cbuf0_size - cbuf->cso[0].user_buffer_size);
+            }
+
+            pcb += cbuf0_size;
+         }
+
+         if (clip_state_size)
+            memcpy(pcb, &ilo->clip, clip_state_size);
+
+         session->pcb_state_vs_changed = true;
       }
-      else {
-         p->state.vs.PUSH_CONSTANT_BUFFER_size = 0;
+      else if (p->state.vs.PUSH_CONSTANT_BUFFER_size) {
          p->state.vs.PUSH_CONSTANT_BUFFER = 0;
-      }
+         p->state.vs.PUSH_CONSTANT_BUFFER_size = 0;
 
-      session->pcb_state_vs_changed = true;
+         session->pcb_state_vs_changed = true;
+      }
+   }
+
+   /* push constant buffer for FS */
+   if (DIRTY(FS) || DIRTY(CBUF)) {
+      const int cbuf0_size = (ilo->fs) ?
+         ilo_shader_get_kernel_param(ilo->fs, ILO_KERNEL_PCB_CBUF0_SIZE) : 0;
+
+      if (cbuf0_size) {
+         const struct ilo_cbuf_state *cbuf = &ilo->cbuf[PIPE_SHADER_FRAGMENT];
+         void *pcb;
+
+         p->state.wm.PUSH_CONSTANT_BUFFER =
+            gen6_emit_push_constant_buffer(p->dev, cbuf0_size, &pcb, p->cp);
+         p->state.wm.PUSH_CONSTANT_BUFFER_size = cbuf0_size;
+
+         if (cbuf0_size <= cbuf->cso[0].user_buffer_size) {
+            memcpy(pcb, cbuf->cso[0].user_buffer, cbuf0_size);
+         }
+         else {
+            memcpy(pcb, cbuf->cso[0].user_buffer,
+                  cbuf->cso[0].user_buffer_size);
+            memset(pcb + cbuf->cso[0].user_buffer_size, 0,
+                  cbuf0_size - cbuf->cso[0].user_buffer_size);
+         }
+
+         session->pcb_state_fs_changed = true;
+      }
+      else if (p->state.wm.PUSH_CONSTANT_BUFFER_size) {
+         p->state.wm.PUSH_CONSTANT_BUFFER = 0;
+         p->state.wm.PUSH_CONSTANT_BUFFER_size = 0;
+
+         session->pcb_state_fs_changed = true;
+      }
    }
 }
 
@@ -1380,7 +1448,7 @@ ilo_3d_pipeline_emit_flush_gen6(struct ilo_3d_pipeline *p)
    if (p->dev->gen == ILO_GEN(6))
       gen6_wa_pipe_control_post_sync(p, false);
 
-   p->gen6_PIPE_CONTROL(p->dev,
+   gen6_emit_PIPE_CONTROL(p->dev,
          PIPE_CONTROL_INSTRUCTION_FLUSH |
          PIPE_CONTROL_WRITE_FLUSH |
          PIPE_CONTROL_DEPTH_CACHE_FLUSH |
@@ -1398,7 +1466,7 @@ ilo_3d_pipeline_emit_write_timestamp_gen6(struct ilo_3d_pipeline *p,
    if (p->dev->gen == ILO_GEN(6))
       gen6_wa_pipe_control_post_sync(p, true);
 
-   p->gen6_PIPE_CONTROL(p->dev,
+   gen6_emit_PIPE_CONTROL(p->dev,
          PIPE_CONTROL_WRITE_TIMESTAMP,
          bo, index * sizeof(uint64_t) | PIPE_CONTROL_GLOBAL_GTT_WRITE,
          true, p->cp);
@@ -1411,7 +1479,7 @@ ilo_3d_pipeline_emit_write_depth_count_gen6(struct ilo_3d_pipeline *p,
    if (p->dev->gen == ILO_GEN(6))
       gen6_wa_pipe_control_post_sync(p, false);
 
-   p->gen6_PIPE_CONTROL(p->dev,
+   gen6_emit_PIPE_CONTROL(p->dev,
          PIPE_CONTROL_DEPTH_STALL |
          PIPE_CONTROL_WRITE_DEPTH_COUNT,
          bo, index * sizeof(uint64_t) | PIPE_CONTROL_GLOBAL_GTT_WRITE,
@@ -1420,7 +1488,6 @@ ilo_3d_pipeline_emit_write_depth_count_gen6(struct ilo_3d_pipeline *p,
 
 static int
 gen6_pipeline_estimate_commands(const struct ilo_3d_pipeline *p,
-                                const struct ilo_gpe_gen6 *gen6,
                                 const struct ilo_context *ilo)
 {
    static int size;
@@ -1466,7 +1533,7 @@ gen6_pipeline_estimate_commands(const struct ilo_3d_pipeline *p,
       }
 
       if (count)
-         size += gen6->estimate_command_size(p->dev, cmd, count);
+         size += ilo_gpe_gen6_estimate_command_size(p->dev, cmd, count);
    }
 
    return size;
@@ -1474,7 +1541,6 @@ gen6_pipeline_estimate_commands(const struct ilo_3d_pipeline *p,
 
 static int
 gen6_pipeline_estimate_states(const struct ilo_3d_pipeline *p,
-                              const struct ilo_gpe_gen6 *gen6,
                               const struct ilo_context *ilo)
 {
    static int static_size;
@@ -1503,7 +1569,7 @@ gen6_pipeline_estimate_states(const struct ilo_3d_pipeline *p,
       int i;
 
       for (i = 0; i < Elements(static_states); i++) {
-         static_size += gen6->estimate_state_size(p->dev,
+         static_size += ilo_gpe_gen6_estimate_state_size(p->dev,
                static_states[i].state,
                static_states[i].count);
       }
@@ -1538,7 +1604,7 @@ gen6_pipeline_estimate_states(const struct ilo_3d_pipeline *p,
    }
 
    if (count) {
-      size += gen6->estimate_state_size(p->dev,
+      size += ilo_gpe_gen6_estimate_state_size(p->dev,
             ILO_GPE_GEN6_SURFACE_STATE, count);
    }
 
@@ -1546,21 +1612,31 @@ gen6_pipeline_estimate_states(const struct ilo_3d_pipeline *p,
    for (shader_type = 0; shader_type < PIPE_SHADER_TYPES; shader_type++) {
       count = ilo->sampler[shader_type].count;
       if (count) {
-         size += gen6->estimate_state_size(p->dev,
+         size += ilo_gpe_gen6_estimate_state_size(p->dev,
                ILO_GPE_GEN6_SAMPLER_BORDER_COLOR_STATE, count);
-         size += gen6->estimate_state_size(p->dev,
+         size += ilo_gpe_gen6_estimate_state_size(p->dev,
                ILO_GPE_GEN6_SAMPLER_STATE, count);
       }
    }
 
    /* pcb (vs) */
-   if (ilo->vs &&
-       ilo_shader_get_kernel_param(ilo->vs, ILO_KERNEL_VS_PCB_UCP_SIZE)) {
-      const int pcb_size =
+   if (ilo->vs) {
+      const int cbuf0_size =
+         ilo_shader_get_kernel_param(ilo->vs, ILO_KERNEL_PCB_CBUF0_SIZE);
+      const int ucp_size =
          ilo_shader_get_kernel_param(ilo->vs, ILO_KERNEL_VS_PCB_UCP_SIZE);
 
-      size += gen6->estimate_state_size(p->dev,
-            ILO_GPE_GEN6_PUSH_CONSTANT_BUFFER, pcb_size);
+      size += ilo_gpe_gen6_estimate_state_size(p->dev,
+            ILO_GPE_GEN6_PUSH_CONSTANT_BUFFER, cbuf0_size + ucp_size);
+   }
+
+   /* pcb (fs) */
+   if (ilo->fs) {
+      const int cbuf0_size =
+         ilo_shader_get_kernel_param(ilo->fs, ILO_KERNEL_PCB_CBUF0_SIZE);
+
+      size += ilo_gpe_gen6_estimate_state_size(p->dev,
+            ILO_GPE_GEN6_PUSH_CONSTANT_BUFFER, cbuf0_size);
    }
 
    return size;
@@ -1571,7 +1647,6 @@ ilo_3d_pipeline_estimate_size_gen6(struct ilo_3d_pipeline *p,
                                    enum ilo_3d_pipeline_action action,
                                    const void *arg)
 {
-   const struct ilo_gpe_gen6 *gen6 = ilo_gpe_gen6_get();
    int size;
 
    switch (action) {
@@ -1579,20 +1654,20 @@ ilo_3d_pipeline_estimate_size_gen6(struct ilo_3d_pipeline *p,
       {
          const struct ilo_context *ilo = arg;
 
-         size = gen6_pipeline_estimate_commands(p, gen6, ilo) +
-            gen6_pipeline_estimate_states(p, gen6, ilo);
+         size = gen6_pipeline_estimate_commands(p, ilo) +
+            gen6_pipeline_estimate_states(p, ilo);
       }
       break;
    case ILO_3D_PIPELINE_FLUSH:
-      size = gen6->estimate_command_size(p->dev,
+      size = ilo_gpe_gen6_estimate_command_size(p->dev,
             ILO_GPE_GEN6_PIPE_CONTROL, 1) * 3;
       break;
    case ILO_3D_PIPELINE_WRITE_TIMESTAMP:
-      size = gen6->estimate_command_size(p->dev,
+      size = ilo_gpe_gen6_estimate_command_size(p->dev,
             ILO_GPE_GEN6_PIPE_CONTROL, 1) * 2;
       break;
    case ILO_3D_PIPELINE_WRITE_DEPTH_COUNT:
-      size = gen6->estimate_command_size(p->dev,
+      size = ilo_gpe_gen6_estimate_command_size(p->dev,
             ILO_GPE_GEN6_PIPE_CONTROL, 1) * 3;
       break;
    default:
@@ -1607,64 +1682,9 @@ ilo_3d_pipeline_estimate_size_gen6(struct ilo_3d_pipeline *p,
 void
 ilo_3d_pipeline_init_gen6(struct ilo_3d_pipeline *p)
 {
-   const struct ilo_gpe_gen6 *gen6 = ilo_gpe_gen6_get();
-
    p->estimate_size = ilo_3d_pipeline_estimate_size_gen6;
    p->emit_draw = ilo_3d_pipeline_emit_draw_gen6;
    p->emit_flush = ilo_3d_pipeline_emit_flush_gen6;
    p->emit_write_timestamp = ilo_3d_pipeline_emit_write_timestamp_gen6;
    p->emit_write_depth_count = ilo_3d_pipeline_emit_write_depth_count_gen6;
-
-#define GEN6_USE(p, name, from) \
-   p->gen6_ ## name = from->emit_ ## name
-   GEN6_USE(p, STATE_BASE_ADDRESS, gen6);
-   GEN6_USE(p, STATE_SIP, gen6);
-   GEN6_USE(p, PIPELINE_SELECT, gen6);
-   GEN6_USE(p, 3DSTATE_BINDING_TABLE_POINTERS, gen6);
-   GEN6_USE(p, 3DSTATE_SAMPLER_STATE_POINTERS, gen6);
-   GEN6_USE(p, 3DSTATE_URB, gen6);
-   GEN6_USE(p, 3DSTATE_VERTEX_BUFFERS, gen6);
-   GEN6_USE(p, 3DSTATE_VERTEX_ELEMENTS, gen6);
-   GEN6_USE(p, 3DSTATE_INDEX_BUFFER, gen6);
-   GEN6_USE(p, 3DSTATE_VF_STATISTICS, gen6);
-   GEN6_USE(p, 3DSTATE_VIEWPORT_STATE_POINTERS, gen6);
-   GEN6_USE(p, 3DSTATE_CC_STATE_POINTERS, gen6);
-   GEN6_USE(p, 3DSTATE_SCISSOR_STATE_POINTERS, gen6);
-   GEN6_USE(p, 3DSTATE_VS, gen6);
-   GEN6_USE(p, 3DSTATE_GS, gen6);
-   GEN6_USE(p, 3DSTATE_CLIP, gen6);
-   GEN6_USE(p, 3DSTATE_SF, gen6);
-   GEN6_USE(p, 3DSTATE_WM, gen6);
-   GEN6_USE(p, 3DSTATE_CONSTANT_VS, gen6);
-   GEN6_USE(p, 3DSTATE_CONSTANT_GS, gen6);
-   GEN6_USE(p, 3DSTATE_CONSTANT_PS, gen6);
-   GEN6_USE(p, 3DSTATE_SAMPLE_MASK, gen6);
-   GEN6_USE(p, 3DSTATE_DRAWING_RECTANGLE, gen6);
-   GEN6_USE(p, 3DSTATE_DEPTH_BUFFER, gen6);
-   GEN6_USE(p, 3DSTATE_POLY_STIPPLE_OFFSET, gen6);
-   GEN6_USE(p, 3DSTATE_POLY_STIPPLE_PATTERN, gen6);
-   GEN6_USE(p, 3DSTATE_LINE_STIPPLE, gen6);
-   GEN6_USE(p, 3DSTATE_AA_LINE_PARAMETERS, gen6);
-   GEN6_USE(p, 3DSTATE_GS_SVB_INDEX, gen6);
-   GEN6_USE(p, 3DSTATE_MULTISAMPLE, gen6);
-   GEN6_USE(p, 3DSTATE_STENCIL_BUFFER, gen6);
-   GEN6_USE(p, 3DSTATE_HIER_DEPTH_BUFFER, gen6);
-   GEN6_USE(p, 3DSTATE_CLEAR_PARAMS, gen6);
-   GEN6_USE(p, PIPE_CONTROL, gen6);
-   GEN6_USE(p, 3DPRIMITIVE, gen6);
-   GEN6_USE(p, INTERFACE_DESCRIPTOR_DATA, gen6);
-   GEN6_USE(p, SF_VIEWPORT, gen6);
-   GEN6_USE(p, CLIP_VIEWPORT, gen6);
-   GEN6_USE(p, CC_VIEWPORT, gen6);
-   GEN6_USE(p, COLOR_CALC_STATE, gen6);
-   GEN6_USE(p, BLEND_STATE, gen6);
-   GEN6_USE(p, DEPTH_STENCIL_STATE, gen6);
-   GEN6_USE(p, SCISSOR_RECT, gen6);
-   GEN6_USE(p, BINDING_TABLE_STATE, gen6);
-   GEN6_USE(p, SURFACE_STATE, gen6);
-   GEN6_USE(p, so_SURFACE_STATE, gen6);
-   GEN6_USE(p, SAMPLER_STATE, gen6);
-   GEN6_USE(p, SAMPLER_BORDER_COLOR_STATE, gen6);
-   GEN6_USE(p, push_constant_buffer, gen6);
-#undef GEN6_USE
 }
