@@ -1,8 +1,8 @@
 /**************************************************************************
- * 
+ *
  * Copyright 2007 Tungsten Graphics, Inc., Cedar Park, Texas.
  * All Rights Reserved.
- * 
+ *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the
  * "Software"), to deal in the Software without restriction, including
@@ -10,11 +10,11 @@
  * distribute, sub license, and/or sell copies of the Software, and to
  * permit persons to whom the Software is furnished to do so, subject to
  * the following conditions:
- * 
+ *
  * The above copyright notice and this permission notice (including the
  * next paragraph) shall be included in all copies or substantial portions
  * of the Software.
- * 
+ *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
  * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
  * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT.
@@ -22,7 +22,7 @@
  * ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
  * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
  * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
- * 
+ *
  **************************************************************************/
 
 
@@ -78,9 +78,10 @@ st_bufferobj_free(struct gl_context *ctx, struct gl_buffer_object *obj)
    assert(obj->RefCount == 0);
    assert(st_obj->transfer == NULL);
 
-   if (st_obj->buffer) 
+   if (st_obj->buffer)
       pipe_resource_reference(&st_obj->buffer, NULL);
 
+   free(st_obj->Base.Label);
    free(st_obj);
 }
 
@@ -174,7 +175,7 @@ st_bufferobj_data(struct gl_context *ctx,
 		  GLenum target,
 		  GLsizeiptrARB size,
 		  const GLvoid * data,
-		  GLenum usage, 
+		  GLenum usage,
 		  struct gl_buffer_object *obj)
 {
    struct st_context *st = st_context(ctx);
@@ -182,10 +183,25 @@ st_bufferobj_data(struct gl_context *ctx,
    struct st_buffer_object *st_obj = st_buffer_object(obj);
    unsigned bind, pipe_usage;
 
+   if (size && data && st_obj->buffer &&
+       st_obj->Base.Size == size && st_obj->Base.Usage == usage) {
+      /* Just discard the old contents and write new data.
+       * This should be the same as creating a new buffer, but we avoid
+       * a lot of validation in Mesa.
+       */
+      struct pipe_box box;
+
+      u_box_1d(0, size, &box);
+      pipe->transfer_inline_write(pipe, st_obj->buffer, 0,
+                                  PIPE_TRANSFER_DISCARD_WHOLE_RESOURCE,
+                                  &box, data, 0, 0);
+      return GL_TRUE;
+   }
+
    st_obj->Base.Size = size;
    st_obj->Base.Usage = usage;
-   
-   switch(target) {
+
+   switch (target) {
    case GL_PIXEL_PACK_BUFFER_ARB:
    case GL_PIXEL_UNPACK_BUFFER_ARB:
       bind = PIPE_BIND_RENDER_TARGET | PIPE_BIND_SAMPLER_VIEW;
@@ -232,7 +248,7 @@ st_bufferobj_data(struct gl_context *ctx,
    pipe_resource_reference( &st_obj->buffer, NULL );
 
    if (ST_DEBUG & DEBUG_BUFFER) {
-      debug_printf("Create buffer size %lu bind 0x%x\n", size, bind);
+      debug_printf("Create buffer size %td bind 0x%x\n", size, bind);
    }
 
    if (size != 0) {
@@ -247,9 +263,11 @@ st_bufferobj_data(struct gl_context *ctx,
 
       if (data)
          pipe_buffer_write(pipe, st_obj->buffer, 0, size, data);
-      return GL_TRUE;
    }
 
+   /* BufferData may change an array or uniform buffer, need to update it */
+   st->dirty.st |= ST_NEW_VERTEX_ARRAYS | ST_NEW_UNIFORM_BUFFER;
+
    return GL_TRUE;
 }
 
@@ -284,7 +302,7 @@ st_bufferobj_map_range(struct gl_context *ctx,
       else
          flags |= PIPE_TRANSFER_DISCARD_RANGE;
    }
-   
+
    if (access & GL_MAP_UNSYNCHRONIZED_BIT)
       flags |= PIPE_TRANSFER_UNSYNCHRONIZED;
 
@@ -330,11 +348,11 @@ st_bufferobj_flush_mapped_range(struct gl_context *ctx,
    assert(length >= 0);
    assert(offset + length <= obj->Length);
    assert(obj->Pointer);
-   
+
    if (!length)
       return;
 
-   pipe_buffer_flush_mapped_range(pipe, st_obj->transfer, 
+   pipe_buffer_flush_mapped_range(pipe, st_obj->transfer,
                                   obj->Offset + offset, length);
 }
 
@@ -374,7 +392,7 @@ st_copy_buffer_subdata(struct gl_context *ctx,
    struct st_buffer_object *dstObj = st_buffer_object(dst);
    struct pipe_box box;
 
-   if(!size)
+   if (!size)
       return;
 
    /* buffer should not already be mapped */
