@@ -256,7 +256,10 @@ void st_init_limits(struct st_context *st)
    c->MaxVarying = screen->get_shader_param(screen, PIPE_SHADER_FRAGMENT,
                                             PIPE_SHADER_CAP_MAX_INPUTS);
    c->MaxVarying = MIN2(c->MaxVarying, MAX_VARYING);
-   c->MaxVaryingComponents = c->MaxVarying * 4;
+   c->FragmentProgram.MaxInputComponents = c->MaxVarying * 4;
+   c->VertexProgram.MaxOutputComponents = c->MaxVarying * 4;
+   c->GeometryProgram.MaxInputComponents = c->MaxVarying * 4;
+   c->GeometryProgram.MaxOutputComponents = c->MaxVarying * 4;
 
    c->MinProgramTexelOffset = screen->get_param(screen, PIPE_CAP_MIN_TEXEL_OFFSET);
    c->MaxProgramTexelOffset = screen->get_param(screen, PIPE_CAP_MAX_TEXEL_OFFSET);
@@ -381,6 +384,7 @@ void st_init_extensions(struct st_context *st)
       { o(ARB_shader_stencil_export),        PIPE_CAP_SHADER_STENCIL_EXPORT            },
       { o(ARB_shader_texture_lod),           PIPE_CAP_SM3                              },
       { o(ARB_shadow),                       PIPE_CAP_TEXTURE_SHADOW_MAP               },
+      { o(ARB_texture_mirror_clamp_to_edge), PIPE_CAP_TEXTURE_MIRROR_CLAMP             },
       { o(ARB_texture_non_power_of_two),     PIPE_CAP_NPOT_TEXTURES                    },
       { o(ARB_timer_query),                  PIPE_CAP_QUERY_TIMESTAMP                  },
       { o(ARB_transform_feedback2),          PIPE_CAP_STREAM_OUTPUT_PAUSE_RESUME       },
@@ -441,8 +445,7 @@ void st_init_extensions(struct st_context *st)
         { PIPE_FORMAT_Z32_FLOAT,
           PIPE_FORMAT_Z32_FLOAT_S8X24_UINT } },
 
-      { { o(ARB_framebuffer_object),
-          o(EXT_packed_depth_stencil) },
+      { { o(EXT_packed_depth_stencil) },
         { PIPE_FORMAT_S8_UINT_Z24_UNORM,
           PIPE_FORMAT_Z24_UNORM_S8_UINT },
         GL_TRUE }, /* at least one format must be supported */
@@ -504,6 +507,8 @@ void st_init_extensions(struct st_context *st)
           PIPE_FORMAT_B10G10R10A2_USCALED,
           PIPE_FORMAT_R10G10B10A2_SSCALED,
           PIPE_FORMAT_B10G10R10A2_SSCALED } },
+      { { o(ARB_vertex_type_10f_11f_11f_rev) },
+        { PIPE_FORMAT_R11G11B10_FLOAT } },
    };
 
    static const struct st_extension_format_mapping tbo_rgb32[] = {
@@ -561,6 +566,7 @@ void st_init_extensions(struct st_context *st)
    ctx->Extensions.NV_fog_distance = GL_TRUE;
    ctx->Extensions.NV_texture_env_combine4 = GL_TRUE;
    ctx->Extensions.NV_texture_rectangle = GL_TRUE;
+   ctx->Extensions.NV_vdpau_interop = GL_TRUE;
 
    ctx->Extensions.OES_EGL_image = GL_TRUE;
    ctx->Extensions.OES_EGL_image_external = GL_TRUE;
@@ -710,6 +716,7 @@ void st_init_extensions(struct st_context *st)
    }
    else if (ctx->Const.MaxSamples >= 2) {
       ctx->Extensions.EXT_framebuffer_multisample = GL_TRUE;
+      ctx->Extensions.EXT_framebuffer_multisample_blit_scaled = GL_TRUE;
    }
 
    if (ctx->Const.MaxDualSourceDrawBuffers > 0 &&
@@ -755,6 +762,10 @@ void st_init_extensions(struct st_context *st)
                              PIPE_BUFFER, PIPE_BIND_SAMPLER_VIEW);
    }
 
+   if (screen->get_param(screen, PIPE_CAP_MIXED_FRAMEBUFFER_SIZES) &&
+       ctx->Extensions.EXT_packed_depth_stencil) {
+      ctx->Extensions.ARB_framebuffer_object = GL_TRUE;
+   }
 
    /* Unpacking a varying in the fragment shader costs 1 texture indirection.
     * If the number of available texture indirections is very limited, then we
